/////////////////////////////////////////////////////////////////////////////////////////////
//
// Name:		WinMain.cpp
//
// Author:		Raymond Wilson - ray.wilson@blueyonder.co.uk
//
// Date:		26th May 2003
//
// Notice:		Copyright (C) 2003, Raymond Wilson
//
/////////////////////////////////////////////////////////////////////////////////////////////

// INCLUDES
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <shlobj.h>
#include "resource.h"
#include "pak.h"

// PROTOTYPES
BOOL CALLBACK DlgProcMain( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam );

// GLOBAL VARIABLES
CPakFile	g_PakFile;				// The PAK file instance

/////////////////////////////////////////////////////////////////////////////////////////////

int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow )
{
	// Launch the dialog box ( DialogBox() is blocking but CreateDialog() is not )
	DialogBox( hInstance, MAKEINTRESOURCE(IDD_MAIN), NULL, (DLGPROC)DlgProcMain );

	// Finish
	return 0;
}

BOOL CALLBACK DlgProcMain( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	// Declare local variables
	char			Buffer1[300] = {0};	// Temporay storage buffer 1
	char			Buffer2[300] = {0};	// Temporay storage buffer 2
	OPENFILENAME	PAKFile;			// For getting the PAK file path and name (to save)	
	BROWSEINFO		CompileFolder;		// Folder that contains PAK contents to compile	
	LPITEMIDLIST	IDFolderSelected;	// Used to get the the compile folders ID

	switch( uMsg )
	{
	case WM_COMMAND:
		switch( LOWORD(wParam) )
		{
		case IDC_BUTTONPATH:
			// Zero the BROWSEINFO and ITEMIDLIST structures and buffer
			ZeroMemory( &Buffer1, 300 );
			ZeroMemory( &CompileFolder, sizeof(BROWSEINFO) );
			ZeroMemory( &IDFolderSelected, sizeof(LPITEMIDLIST) );

			// Fill in the needed BROWSEINFO structure elemens
			CompileFolder.lpszTitle = "Please Select Folder";	
			
			// Call the dialog to browse for a folder
			IDFolderSelected = SHBrowseForFolder( &CompileFolder );

			// Get the selected path
			SHGetPathFromIDList( IDFolderSelected, Buffer1 );

			// Set the corresponding edit box to contain the absolute path
			// for the folder containing files to compile in to the PAK
			SetDlgItemText( hDlg, IDC_COMPILATION, Buffer1 );

			break;
		case IDC_BUTTONFILE:
			// Zero the OPENFILENAME structure and buffer
			ZeroMemory( &PAKFile, sizeof(OPENFILENAME) );
			ZeroMemory( &Buffer1, 300 );

			// Fill in the file structure
			PAKFile.lStructSize		= sizeof(OPENFILENAME);
			PAKFile.lpstrFile		= Buffer1;
			PAKFile.nMaxFile		= MAX_PATH;
			PAKFile.Flags			= OFN_PATHMUSTEXIST;
			PAKFile.lpstrFilter		= "PAK Files (*.pak)\0*.pak\0\0";
			PAKFile.lpstrDefExt		= "pak";
			PAKFile.nFilterIndex	= 1;

			// Launch the dialog and get the PAK path / name
			GetSaveFileName( &PAKFile );
			
			// Set the corresponding edit box to contain the absolute path
			// for the file to create
			SetDlgItemText( hDlg, IDC_OUTPUTFILE, Buffer1 );

			break;
		case IDCOMPILE:
			// Clear Buffers
			ZeroMemory( Buffer1, 300 );
			ZeroMemory( Buffer2, 300 );

			// Get the compilation folder path and output file name
			GetDlgItemText( hDlg, IDC_COMPILATION, Buffer1, 300 );
			GetDlgItemText( hDlg, IDC_OUTPUTFILE, Buffer2, 300 );

			// If compilation folder is empty - display a message and break
			if( strlen(Buffer1) == 0 )
			{ 
				MessageBox( NULL, "Please select the compilation folder.", "Note:", MB_OK ); 
				break;
			}

			// If output file is empty - display a message and break
			if( strlen(Buffer2) == 0 )
			{ 
				MessageBox( NULL, "Please select an output file and location.", "Note:", MB_OK ); 
				break;
			}
			
			// Create the PAK file
			if( !g_PakFile.CreatePak( Buffer1, Buffer2 ) )
				// Error
				MessageBox( NULL, "There was a compilation error.", "Note:", MB_OK );
			else
				MessageBox( NULL, "PAK compiled successfully.", "Note:", MB_OK );

			break;
		case IDCANCEL:
			EndDialog( hDlg, TRUE );
			break;
		}
		break;
	case WM_CLOSE:
	case WM_DESTROY:
		EndDialog( hDlg, TRUE );
		break;
	}

	return FALSE;
}